@@ -2,21 +2,26 @@
 #'
 #' Function to preprocess your data for input into [run_ml()].
 #'
+#' @inheritParams group_correlated_features
+#'
+#' @inheritParams run_ml
+#' @inheritParams group_correlated_features
+#'
 #' @param method Methods to preprocess the data, described in
 #'   [caret::preProcess()] (default: `c("center","scale")`, use `NULL` for
 #'   no normalization).
 #' @param remove_var Whether to remove variables with near-zero variance
 #'   (`'nzv'`; default), zero variance (`'zv'`), or none (`NULL`).
-#' @param collapse_corr_feats Whether to keep only one of perfectly correlated
-#'   features.
+#' @param collapse_corr_feats Whether to keep only one of correlated features
+#'   (see `corr_method` and `corr_thresh`)
 #' @param to_numeric Whether to change features to numeric where possible.
 #' @param prefilter_threshold Remove features which only have non-zero & non-NA
-#'   values N rows or fewer (default: 1). Set this to -1 to keep all columns at
-#'   this step. This step will also be skipped if `to_numeric` is set to
+#'   values in N rows or fewer (default: 1). Set this to -1 to keep all columns
+#'   at this step. This step will also be skipped if `to_numeric` is set to
 #'   `FALSE`.
-#' @inheritParams run_ml
-#' @inheritParams group_correlated_features
-#'
+#' @param name Name of results used when the input is
+#'   `TreeSummarizedExperiment`. This same name is used for `assay` and
+#'   `altExp`.
 #'
 #' @return
 #'
@@ -25,6 +30,11 @@
 #' - `grp_feats`: If features were grouped together, a named list of the features corresponding to each group.
 #' - `removed_feats`: Any features that were removed during preprocessing (e.g. because there was zero variance or near-zero variance for those features).
 #'
+#' If the input is `TreeSummarizedExperiment`, the output is added as an
+#' additional data to the input object. If the set of features match in output
+#' and input, the results are stored directly to `assay` slot. If they
+#' do not match, the output is stored to `altExp` slot of the object.
+#'
 #' If the `progressr` package is installed, a progress bar with time elapsed
 #' and estimated time to completion can be displayed.
 #'
@@ -53,14 +63,127 @@
 #' ## tell progressor to always report progress
 #' \dontrun{
 #' progressr::handlers(global = TRUE)
-#' ## run the function and watch the live progress udpates
+#' ## run the function and watch the live progress updates
 #' dat_preproc <- preprocess_data(mikropml::otu_small, "dx")
+#'
+#' # Create TreeSE object
+#' library(TreeSummarizedExperiment)
+#' df <- mikropml::otu_small
+#' assay <- df[, !colnames(df) %in% c("dx"), drop = FALSE] |> t() |> as.matrix()
+#' tse <- TreeSummarizedExperiment(assays = SimpleList(counts = assay))
+#' colData(tse)[["dx"]] <- df[["dx"]]
+#'
+#' # Preprocess
+#' tse <- preprocess_data(
+#'   dataset = tse,
+#'   assay.type = "counts",
+#'   outcome_colname = "dx"
+#' )
+#' # The result is in assay slot
+#' tse
 #' }
-preprocess_data <- function(dataset, outcome_colname,
-                            method = c("center", "scale"),
-                            remove_var = "nzv", collapse_corr_feats = TRUE,
-                            to_numeric = TRUE, group_neg_corr = TRUE,
-                            prefilter_threshold = 1) {
+#'
+#' @rdname preprocess_data
+#' @export
+methods::setGeneric("preprocess_data", signature = "dataset", function(dataset, ...)
+  standardGeneric("preprocess_data"))
+
+#' @rdname preprocess_data
+#' @export
+#' @importFrom SummarizedExperiment assayNames assay colData
+#' @importFrom SingleCellExperiment altExpNames altExp
+methods::setMethod("preprocess_data", signature = c(dataset = "TreeSummarizedExperiment"),
+    function(dataset, outcome_colname, assay.type = "counts", col.var = NULL,
+             altexp = NULL, name = "preprocessed", ...){
+  if( !(is.null(altexp) || (is.character(altexp) &&
+      length(altexp) == 1L && altexp %in% altExpNames(dataset)) ) ){
+    stop("'altexp' must be NULL or specify alternative experiment from ",
+         "altExpNames(x).", call. = FALSE)
+  }
+  if( !is.null(altexp) ){
+    dataset <- altExp(dataset, altexp)
+  }
+  if( !(is.character(assay.type) && length(assay.type) == 1L &&
+        assay.type %in% assayNames(dataset) ) ){
+    stop("'assay.type' must specify assay from assayNames(x).", call. = FALSE)
+  }
+  if( !(is.character(outcome_colname) && length(outcome_colname) == 1L &&
+        outcome_colname %in% colnames(colData(dataset)) ) ){
+    stop("'outcome_colname' must specify column from colData(x).", call. = FALSE)
+  }
+  if( !(is.null(col.var) ||
+        (is.character(col.var) && all(col.var %in% colnames(colData(dataset)))) ) ){
+    stop("'col.var' must be NULL or specify columns from colData(x).", call. = FALSE)
+  }
+  if( !(is.character(name) && length(name) == 1L ) ){
+    stop("'name' must be single character value.", call. = FALSE)
+  }
+  # Get assay and specified columns
+  mat <- assay(dataset, assay.type) |> t()
+  col <- colData(dataset)[ , c(outcome_colname, col.var), drop = FALSE]
+  df <- cbind(mat, col) |> as.data.frame()
+  # Preprocess data
+  res <- preprocess_data(dataset = df, outcome_colname = outcome_colname, ...)
+  # Add data back to SE
+  dataset <- add_data_to_se(dataset, res, outcome_colname, name)
+  return(dataset)
+  }
+)
+
+#' @importFrom S4Vectors SimpleList DataFrame metadata
+#' @importFrom SummarizedExperiment assay `assay<-` `metadata<-`
+#' @importFrom TreeSummarizedExperiment TreeSummarizedExperiment
+#' @importFrom SingleCellExperiment altExp<-
+#' @importFrom stats setNames
+add_data_to_se <- function(tse, res, outcome_colname, name){
+  # Remove samples if the outcome value was missing
+  if( nrow(res[[1L]]) != ncol(tse) ){
+    tse <- tse[ , !is.na(tse[[ outcome_colname ]])]
+  }
+  # Get assay data
+  mat <- res[[1]]
+  mat <- mat[ , !colnames(mat) %in% outcome_colname, drop = FALSE] |> t()
+  # If the features do not match, add preprocessed data to altExp
+  if( length(setdiff( rownames(mat), rownames(tse))) > 0L ){
+    message("Adding preprocessed data to altExp(dataset, '", name, "').")
+    # Create new TreeSE
+    assays <- setNames(SimpleList(mat), name)
+    # If features were grouped, we add the grouping info to rowData
+    rd <- NULL
+    if( !is.null(res[[2]]) ){
+      rd <- res[[2]]
+      rd <- rd[ match(rownames(mat), names(rd)) ]
+      rd <- rd |> I() |> DataFrame()
+      colnames(rd) <- names(res)[[2]]
+      rownames(rd) <- rownames(mat)
+    }
+    # Create TreeSE
+    tse_add <- TreeSummarizedExperiment(
+      assays = assays, colData = colData(tse), rowData = rd)
+    # Add info on remvoed featrues to metadata
+    metadata(tse_add)[["removed_feats"]] <- res[["removed_feats"]]
+    # Add new TreeSE to altExp of old one
+    altExp(tse, name) <- tse_add
+  } else{
+    # If all features match, we can add the matrix back to original TreeSE as
+    # new assay
+    mat <- mat[rownames(tse), , drop = FALSE]
+    assay(tse, name, withDimnames = FALSE) <- mat
+    metadata(tse)[["removed_feats"]] <- res[["removed_feats"]]
+  }
+  return(tse)
+}
+
+#' @rdname preprocess_data
+#' @export
+methods::setMethod("preprocess_data", signature = c(dataset = "ANY"),
+    function(dataset, outcome_colname,
+             method = c("center", "scale"),
+             remove_var = "nzv", collapse_corr_feats = TRUE,
+             corr_method = "spearman",
+             corr_thresh = 1,
+             to_numeric = TRUE, group_neg_corr = TRUE,
+             prefilter_threshold = 1, ...) {
   progbar <- NULL
   if (isTRUE(check_packages_installed("progressr"))) {
     progbar <- progressr::progressor(steps = 20, message = "preprocessing")
@@ -116,6 +239,8 @@ preprocess_data <- function(dataset, outcome_colname,
       pbtick(progbar)
     }
     feats_and_grps <- collapse_correlated_features(processed_feats,
+      corr_method = corr_method,
+      corr_thresh = corr_thresh,
       group_neg_corr,
       progbar = progbar
     )
@@ -133,7 +258,7 @@ preprocess_data <- function(dataset, outcome_colname,
     grp_feats = grp_feats,
     removed_feats = removed_feats
   ))
-}
+})
 
 #' Remove missing outcome values
 #'
@@ -477,6 +602,7 @@ get_caret_dummyvars_df <- function(features, full_rank = FALSE, progbar = NULL)
 
 
 #' Collapse correlated features
+#'
 #' @inheritParams process_novar_feats
 #' @inheritParams group_correlated_features
 #'
@@ -488,14 +614,17 @@ get_caret_dummyvars_df <- function(features, full_rank = FALSE, progbar = NULL)
 #' \dontrun{
 #' collapse_correlated_features(mikropml::otu_small[, 2:ncol(otu_small)])
 #' }
-collapse_correlated_features <- function(features, group_neg_corr = TRUE, progbar = NULL) {
+collapse_correlated_features <- function(features, group_neg_corr = TRUE,
+                                         corr_method = "spearman",
+                                         corr_thresh = 1,
+                                         progbar = NULL) {
   feats_nocorr <- features
   grp_feats <- NULL
   if (!is.null(features)) {
     sapply_fn <- select_apply(fun = "sapply")
     if (any(sapply_fn(features, class) %in% c("character", "factor"))) {
       stop(
-        "Some features are charactors or factors. Please remove these before proceeding with `collapse_correlated_features`."
+        "Some features are characters or factors. Please remove these before proceeding with `collapse_correlated_features`."
       )
     }
     if (!is.null(process_novar_feats(features, progbar = progbar)$novar_feats)) {
@@ -505,6 +634,8 @@ collapse_correlated_features <- function(features, group_neg_corr = TRUE, progba
     }
     if (ncol(features) != 1) {
       corr_feats <- group_correlated_features(features,
+        corr_method = corr_method,
+        corr_thresh = corr_thresh,
         group_neg_corr = group_neg_corr
       )
       corr_cols <- gsub("\\|.*", "", corr_feats)
