@@ -283,6 +283,43 @@ test_that("preprocess_data works", {
       "var11"
     ))
   )) %>% suppressMessages()
+  expect_equal(
+    preprocess_data(
+      test_df,
+      "outcome",
+      corr_thresh = .8,
+      corr_method = "pearson",
+      prefilter_threshold = -1
+    ),
+    list(
+      dat_transformed = structure(
+        list(
+          outcome = c("normal", "normal", "cancer"),
+          grp1 = c(-1, 0, 1)
+        ),
+        class = c("tbl_df", "tbl", "data.frame"),
+        row.names = c(NA, -3L)
+      ),
+      grp_feats = list(
+        grp1 = c(
+          "var1",
+          "var10_0",
+          "var10_1",
+          "var12",
+          "var2_a",
+          "var2_b",
+          "var2_c",
+          "var3_yes",
+          "var4_1",
+          "var7_1",
+          "var8",
+          "var9_x",
+          "var9_y"
+        )
+      ),
+      removed_feats = c("var5", "var6", "var11")
+    )
+  ) %>% suppressMessages()
 })
 
 test_that("rm_missing_outcome works", {
@@ -575,7 +612,7 @@ test_that("collapse_correlated_features works", {
   ) %>% suppressMessages()
   expect_error(
     collapse_correlated_features(test_df[1:3, c(2, 5, 6, 7)]),
-    "Some features are charactors or factors. Please remove these before proceeding with `collapse_correlated_features`."
+    "Some features are characters or factors. Please remove these before proceeding with `collapse_correlated_features`."
   ) %>% suppressMessages()
   expect_error(
     collapse_correlated_features(test_df[1:3, c(2, 5, 6, 8)]),
@@ -671,3 +708,51 @@ test_that("preprocess_data replaces spaces in outcome column values (class label
     dat_proc
   ) %>% suppressMessages()
 })
+
+test_that("preprocess_data works with TreeSE", {
+  library(TreeSummarizedExperiment)
+  tse <- makeTSE(nrow = 100, ncol = 50)
+  assayNames(tse) <- "my_assay"
+  # Wrong assay name
+  preprocess_data(dataset = tse, assay.type = "wrong_name", outcome_colname = "group") |> expect_error()
+  # Preprocess data without grouping. The feature set in output should be the same as in input.
+  tse <- preprocess_data(
+    dataset = tse,
+    assay.type = "my_assay",
+    outcome_colname = "group",
+    remove_var = NULL,
+    collapse_corr_feats = FALSE
+  ) |> suppressMessages()
+  expect_s4_class(tse, "TreeSummarizedExperiment")
+  # There should be new assay with same set of features
+  expect_true( "preprocessed" %in% assayNames(tse) )
+
+  # When we group features, the output is stored to altExp
+  tse <- preprocess_data(
+    dataset = tse,
+    assay.type = "my_assay",
+    outcome_colname = "group",
+    name = "output"
+  ) |> suppressMessages()
+  expect_s4_class(tse, "TreeSummarizedExperiment")
+  # There should be new assay in generated altExp
+  expect_true( "output" %in% altExpNames(tse) )
+  expect_true( "output" %in% assayNames(altExp(tse, "output")) )
+
+  # Check that the values match with ones generated with simple data.frame
+  res <- altExp(tse, "output")
+  df <- assay(tse, "my_assay") |> t() |> as.data.frame()
+  df[["outcome"]] <- tse[["group"]]
+  ref <- preprocess_data(
+    dataset = df,
+    outcome_colname = "outcome"
+  ) |> suppressMessages()
+  values_ref <- ref[["dat_transformed"]]
+  values_ref <- values_ref[ , !colnames(values_ref) %in% "outcome", drop = FALSE] |> t() |> as.matrix()
+  values <- assay(res, "output")
+  colnames(values) <- NULL
+  expect_equal(values, values_ref)
+  #
+  expect_equal( metadata(res), ref[ "removed_feats" ] )
+  expect_equal( rowData(res)[["grp_feats"]], ref[["grp_feats" ]] )
+})
